'''
the bugzilla report function module
'''
import sys
import time
import logging
import bugzilla
import tempfile
import aggregate
from ..tools.retrying import retrying, EAgain
from ..work.data import load_yaml, save_result, set_config_filename
from ..work.common import RESULT_PASSED, RESULT_SKIP, RESULT_WAIVED
from result import get_hwp_result
from gevent.pool import Pool
from gevent.coros import RLock
import yaml
try:
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Dumper

logger = logging.getLogger(__name__)
DEFAULT_URL='https://bugzilla.redhat.com/xmlrpc.cgi'
DEFAULT_COMPONENT='images'
DEFAULT_PRODUCT='Cloud Image Validation'
MAXTRIES=40
SLEEP=3


def bugzilla_credentials(configfile):
    '''get bugzilla credentials from a config file'''
    configfile = set_config_filename(configfile)
    config = load_yaml(configfile)
    return config['bugzilla']['user'], config['bugzilla']['password']


@retrying(maxtries=MAXTRIES, sleep=SLEEP)
def connect(url, user, password):
    '''connect to bugzilla, return the bugzilla connection'''
    ret = bugzilla.RHBugzilla(url=url, user=user, password=password)
    if not ret:
        raise RuntimeError("Couldn't connect to bugzilla: %s %s %s" % (url, username, password))
    logger.debug('connected to bugzilla: %s %s %s %s', url, user, password, ret)
    return ret

@retrying(maxtries=MAXTRIES, sleep=SLEEP)
def create_bug(connection, summary, version, arch, component=DEFAULT_COMPONENT, bugzilla_product=DEFAULT_PRODUCT,
    op_sys='Linux', keywords=['TestOnly']):
    '''create particular bug'''
    return connection.createbug(product=bugzilla_product, component=component, version='RHEL' + str(version),
                rep_platform=arch, summary=summary, op_sys=op_sys, keywords=keywords)

@retrying(maxtries=MAXTRIES, sleep=SLEEP, final_exception=AssertionError)
def assert_bug(connection, bug):
    '''assert a bug exists'''
    try:
        return connection.getbug(bug.bug_id)
    except bugzilla.Fault as err:
        raise EAgain(err)

@retrying(maxtries=MAXTRIES, sleep=SLEEP)
def create_bug_log_attachment(connection, bug, ami, data):
    '''create bug attachment from data'''
    with tempfile.NamedTemporaryFile() as fd:
        logger.debug('%s got attachment tmpfile: %s', ami, fd.name)
        fd.write(yaml.dump(data, Dumper=Dumper))
        fd.seek(0)
        assert_bug(connection, bug)
        logger.debug('uploading %s log attachment', ami)
        attach_name = ami + '-log.yaml'
        res = connection.attachfile(bug.bug_id, fd, attach_name, filename=attach_name, contenttype='text/yaml', ispatch=False)
        logger.debug('uploading %s log attachmet done: %s', ami, res)

@retrying(maxtries=MAXTRIES, sleep=SLEEP)
def comment_bug(connection, bug, comment):
    res = assert_bug(connection, bug).addcomment(comment)
    logger.debug('bz %s added comment %s: %s', bug.bug_id, comment, res)


def process_ami_record(ami_key, ami_data, whitelist=[], user=None, password=None,
        url=DEFAULT_URL, component=DEFAULT_COMPONENT, bugzilla_product=DEFAULT_PRODUCT,
        verbose=False):
    '''process one ami record creating bug with comment per hwp'''
    connection = connect(url, user, password)
    summary = " ".join(ami_key)
    region, platform, product, version, arch, itype, ami = ami_key
    bug = create_bug(connection, summary, version, arch, component, bugzilla_product)
    create_bug_log_attachment(connection, bug, ami, ami_data)
    ami_result = RESULT_PASSED
    data = aggregate.nested(ami_data, 'cloudhwname')
    for hwp in data:
        sub_result, sub_log = get_hwp_result(data[hwp], whitelist, verbose)
        if sub_result not in [RESULT_PASSED, RESULT_SKIP] and ami_result == RESULT_PASSED:
            ami_result = sub_result
        bug.addcomment('# %s: %s\n%s' % (hwp, sub_result, '\n'.join(sub_log)))
    bug.setstatus(ami_result == RESULT_PASSED and 'VERIFIED' or 'ON_QA')
    return bug.bug_id, ami, ami_result

def process_ami_record_debug(ami_key, ami_data, whitelist=[], user=None, password=None,
        url=DEFAULT_URL, component=DEFAULT_COMPONENT, bugzilla_product=DEFAULT_PRODUCT,
        verbose=False):
    '''process one ami record creating bug with comment per hwp'''
    summary = " ".join(ami_key)
    ami = ami_key[-1]
    logger.debug('*** Summary for bug: %s', summary)
    logger.debug('*** Ami data: %s', ami_data)
    bug = 'not created'
    ami_result = RESULT_PASSED
    data = aggregate.nested(ami_data, 'cloudhwname')
    for hwp in data:
        sub_result, sub_log = get_hwp_result(data[hwp], whitelist, verbose)
        if sub_result not in [RESULT_PASSED, RESULT_SKIP] and ami_result == RESULT_PASSED:
            ami_result = sub_result
        logger.debug('.....adding comment: # %s: %s\n%s' % (hwp, sub_result, '\n'.join(sub_log)))
    logger.debug(".....setting status: %s", ami_result == RESULT_PASSED and 'VERIFIED' or 'ON_QA')
    return bug, ami, ami_result


def main(config, istream, ostream, test_whitelist, user=None, password=None, url=DEFAULT_URL, component=DEFAULT_COMPONENT,
         bugzilla_product=DEFAULT_PRODUCT, verbose=False, pool_size=128, debug_mode=False):
    user, password = bugzilla_credentials(config)
    logger.debug('got credentials: %s, %s', user, password)
    data = load_yaml(istream)
    whitelist = [str(item) for item in test_whitelist[0].split(',')]
    agg_data = aggregate.flat(data, 'region', 'platform', 'product', 'version', 'arch', 'itype',
                                'ami')
    pool = Pool(size=pool_size)
    if debug_mode:
        #There will be enhancement to have for each ami output in the file
        statuses = pool.map(lambda (key, data): process_ami_record_debug(key, data, whitelist, user=user,
            password=password, url=url, component=component, bugzilla_product=bugzilla_product),
            agg_data.items())
    else:
        statuses = pool.map(lambda (key, data): process_ami_record(key, data, whitelist, user=user,
            password=password, url=url, component=component, bugzilla_product=bugzilla_product),
            agg_data.items())
    for bug, ami, status in statuses:
        save_result(ostream, dict(bug=bug, id=ami, status=status))
    return all([status in (RESULT_PASSED,RESULT_WAIVED) for _, status, _ in statuses]) and 0 or 1




""" This module contains testcase_38_rclocal_checksum test """
from testcase import Testcase


class testcase_38_rclocal_checksum(Testcase):
    """
    Get checksum for /etc/rc.d/rc.local (debugging purposes)
    """
    tags = []
    stages = ['stage1']

    # pylint: disable=W0613
    def test(self, connection, params):
        """ Perform test """

        self.get_result(connection, '[ -f /etc/rc.d/rc.local ] && md5sum /etc/rc.d/rc.local || echo "no rc.local"')
        return self.log



""" This module contains testcase_98_dmesg_stage1 test """
import os
import tempfile
from testcase import Testcase


class testcase_98_dmesg_stage1(Testcase):
    """
    Grab dmesg output
    """
    tags = ['kernel']
    stages = ['stage1']

    # pylint: disable=W0613
    def test(self, connection, params):
        """ Perform test """

        self.get_result(connection, 'dmesg > /tmp/dmesg; echo')
        tfile = tempfile.NamedTemporaryFile(delete=False)
        tfile.close()
        connection.sftp.get('/tmp/dmesg', tfile.name)
        with open(tfile.name, 'r') as dmesgfile:
            dmesg = dmesgfile.read()
        os.unlink(tfile.name)
        self.log.append({'result': 'passed',
                         'comand': 'get dmesg',
                         'dmesg': dmesg
                         })
        return self.log



#!/usr/bin/env python

from setuptools import setup
import glob

setup(name='dva',
    version='0.6',
    description="dva's validation. again.",
    author='dparalen',
    author_email='vetrisko@gmail.com',
    url='https://github.com/RedHatQE/dva',
    license="GPLv3+",
    provides='dva',
    install_requires=['PyYAML', 'aaargh', 'stitches>=0.9', 'boto',
        'gevent', 'html', 'python-bugzilla', 'requests', 'urllib3'],
    packages=[
        'dva', 'dva.test', 'dva.cloud', 'dva.work', 'dva.tools', 'dva.connection', 'dva.report'
        ],
    data_files=[
             ('/usr/share/dva/hwp', glob.glob('hwp/*.yaml')),
             ('/usr/share/dva/data', glob.glob('data/*')),
             ('/usr/share/dva/examples', glob.glob('examples/*.yaml')),
             ('/etc', ['etc/dva.yaml']),
             ('/etc/bash_completion.d/', ['etc/bash_completion.d/dva'])
    ],
    classifiers=[
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'Programming Language :: Python',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Operating System :: POSIX',
            'Intended Audience :: Developers',
            'Development Status :: 4 - Beta'
    ],
    scripts=['scripts/dva'],
)



'''logged decorator module'''
import logging
from functools import wraps
from traceback import extract_stack
from textwrap import wrap
logger = logging.getLogger(__name__)

def logged(logger=logger, level=logging.DEBUG):
    '''the logged decorator'''
    def decorator_wrapper(fn):
        @wraps(fn)
        def wrapper(*args, **kvs):
            ret = None
            if logger.isEnabledFor(level):
                # performance reasons
                # not indent logged, decorator_wrapper, wraps & wrapper
                indent = len(extract_stack()) - 4
                assert indent > 0, 'oOops'
                call_str = '-> %s(*%s, **%s)' % (fn.__name__, args, kvs) 
                for log_line in wrap(call_str, initial_indent=' ' * indent,
                                        subsequent_indent=' ' * indent):
                    logger.log(level, log_line)
                try:
                    ret = fn(*args, **kvs)
                finally:
                    ret_str = '<- %s' % (ret,)
                    for log_line in wrap(ret_str, initial_indent=' ' * indent,
                                            subsequent_indent=' ' * indent):
                        logger.log(level, log_line)
            else:
                # logging not enabled, no need to try:...
                ret = fn(*args, **kvs)
            return ret
        return wrapper
    return decorator_wrapper

__all__ = ['logged']

                



""" This module contains testcase_60_yum_update test """
from testcase import Testcase


class testcase_60_yum_update(Testcase):
    """
    Test system update with 'yum update'
    """

    stages = ['stage1']
    tags = ['default', 'content']
    after = ['testcase_55_yum_group_install']
    applicable = {"platform": "(?i)RHEL|BETA", "version": "OS (>=5.5, !=6.0)"}

    def test(self, connection, params):
        """ Perform test """

        prod = params['platform'].upper()
        ver = params['version']
        if prod in ['RHEL', 'BETA'] and ver.startswith('6.') and params['cloudhwname'] == 't1.micro':
            # Creating swap to workaround mem<1024M issue
            self.ping_pong(connection, 'head -c $((1024*1024*1024)) /dev/zero > /swap && echo SUCCESS', '\r\nSUCCESS\r\n', 150)
            self.get_return_value(connection, 'mkswap /swap', 30)
            self.ping_pong(connection, 'echo \'/swap    swap     swap    defaults     0 0\' >> /etc/fstab && echo SUCCESS', '\r\nSUCCESS\r\n')
            self.get_return_value(connection, 'swapon -a -e', 30)

        # workaround for https://bugzilla.redhat.com/show_bug.cgi?id=912568
        # https://access.redhat.com/site/solutions/175393
        self.get_return_value(connection, 'rpm -q matahari-net && yum -y remove matahari-net ||:', 90)
        if prod in ['RHEL', 'BETA'] and ver.startswith('5.'):
            self.get_return_value(connection, 'yum -y install kernel-xen', 900)
        elif prod == 'FEDORA' and params['arch'] == 'i386':
            self.get_return_value(connection, 'yum -y install kernel-PAE', 900)
        else:
            self.get_return_value(connection, 'yum -y install kernel', 900)
        self.get_return_value(connection, 'yum -y update', 1800)
        if (prod in ['RHEL', 'BETA'] and ver.startswith('5.')) or prod == 'FEDORA':
            # Preparing the latest kernel for stage2 testing
            self.get_return_value(connection, r'sed -i "s,\(default\)=.*$,\1=0," /boot/grub/menu.lst')
        return self.log



'''
a dependencies-handling module
'''

import logging
LOG = logging.getLogger(__name__)
WHITE='white'
GRAY='gray'
BLACK='black'


class Graph(object):
    '''The graph object'''
    def __init__(self, edges=set([]), vertices=set([])):
        self.vertices = vertices
        self.edges = edges

    def __repr__(self):
        return '%s(vertices=%r, edges=%r)' % (type(self).__name__, self.vertices, self.edges)

    def add_edge(self, from_vertex, to_vertex):
        '''add an edge--tuple registering the vertices'''
        self.vertices.add(from_vertex)
        self.vertices.add(to_vertex)
        self.edges.add((from_vertex, to_vertex))

    def add_vertex(self, vertex):
        ''' add a single vertex'''
        self.vertices |= set([vertex])

    def adjacent(self, other):
        '''get set of vertices adjacent to the vertex provided'''
        return set([adj_vertex for vertex, adj_vertex, in self.edges if vertex == other])


class Vertex(object):
    '''a vertex object'''
    def __init__(self, value=None, parent=None, color=WHITE, distance=-1):
        self.value = value
        self.color = color
        self.parent = parent
        self.distance = distance

    def __repr__(self):
            return '%r(value=%r, parent=%r, color=%r, distance=%r)' % (type(self).__name__, self.value, self.parent,
                    self.color, self.distance)

    def __cmp__(self, other):
        return cmp(self.value, other.value)

def bfs(G, root):
    '''
    the bfs algorithm modified to work in etages of the dependency tree built
    etages are being yielded for the caller to process during the graph traversal
    '''
    for vertex in G.vertices - set([root]):
        vertex.color = WHITE
        vertex.distance = -1
        vertex.parent = None

    root.color = GRAY
    root.distance = 0
    root.parent = None

    etage_0 = []
    etage_1 = []
    etage_0.append(root)
    while etage_0:
        yield etage_0
        while etage_0:
            vertex = etage_0.pop(0)
            for adj_vertex in G.adjacent(vertex):
                if adj_vertex.color is not WHITE:
                    continue
                adj_vertex.color = GRAY
                adj_vertex.distance = vertex.distance + 1
                adj_vertex.parent = vertex
                etage_1.append(adj_vertex)
            vertex.color = BLACK
        etage_0, etage_1 = etage_1, etage_0





""" This module contains testcase_28_iptables test """
from testcase import Testcase


class testcase_28_iptables(Testcase):
    """
    Check default iptables setup
    Note that in rhel6.5 iptables are disabled by default and it should be no pre-loaded rules
    """
    stages = ['stage1']
    applicable = {'platform': '(?i)RHEL|BETA', 'version': 'OS (>=5.5, <6.5)'}
    tags = ['default']

    def test(self, connection, params):
        """ Perform test """

        ver = params['version']
        if ver == '6.5':
            # check that there are no pre-loaded rules
            # pylint: disable=C0301
            self.ping_pong(connection, r'iptables -L -n | egrep -v "^Chain .* \(policy ACCEPT\)$|^target.*prot.*opt.*source.*destination|^$" && echo "FAILED" || echo "SUCCESS"',
                           '\r\nSUCCESS\r\n')
        else:
            self.ping_pong(connection, 'iptables -L -n | grep :22 | grep ACCEPT | wc -l', '\r\n1\r\n')
            self.ping_pong(connection, 'iptables -L -n | grep RELATED,ESTABLISHED | grep ACCEPT | wc -l', '\r\n1\r\n')
            if ver.startswith('6.'):
                self.ping_pong(connection, 'iptables -L -n | grep REJECT | grep all | grep 0.0.0.0/0 | grep icmp-host-prohibited | wc -l', '\r\n2\r\n')
            elif ver.startswith('5.'):
                self.ping_pong(connection, 'iptables -L -n | grep :631 | grep ACCEPT | wc -l', '\r\n2\r\n')
                self.ping_pong(connection, 'iptables -L -n | grep :5353 | grep ACCEPT | wc -l', '\r\n1\r\n')
                self.ping_pong(connection, 'iptables -L -n | grep -e esp -e ah | grep ACCEPT | wc -l', '\r\n2\r\n')
                self.ping_pong(connection, 'iptables -L -n | grep REJECT | grep all | grep 0.0.0.0/0 | grep icmp-host-prohibited | wc -l', '\r\n1\r\n')
        return self.log



""" This module contains testcase_26_verify_rpms test """
import yaml
from testcase import Testcase
from dva.work.common import RESULT_FAILED


class testcase_26_verify_rpms(Testcase):
    """
    Do rpm -Va and compare the number of packages with modified files
    """
    stages = ['stage1']
    tags = ['default']

    def test(self, connection, params):
        """ Perform test """

        plat = params['platform'].upper()
        prod = params['product'].upper()
        ver = params['version']

        if connection.rpyc is None:
            self.log.append({
                'result': RESULT_FAILED,
                'comment': 'test can\'t be performed without RPyC connection'})
            return self.log

        modified_files = {}

        cmd = ("/bin/sh", "-c", "rpm -Va --nomtime --nosize --nomd5; exit 0")
        if list(connection.rpyc.modules.sys.version_info)[1] >= 6:
            # python 2.6 - 2.7
            cmd = list(cmd)

        proc = connection.rpyc.modules.subprocess.Popen(cmd, stdout=connection.rpyc.modules.subprocess.PIPE, stderr=connection.rpyc.modules.subprocess.PIPE)
        out, _ = proc.communicate()
        if proc.returncode != 0:
            self.log.append({
                'result': RESULT_FAILED,
                'comment': 'failed to get rpm -Va result from host, error %s' % proc.returncode})
            return self.log

        files = out.split('\n')

        for fline in files:
            pos = fline.find('/')
            if pos != -1:
                modified_files[fline[pos:]] = True

        with open(self.datadir + '/verify_rpms.yaml') as expected_modified_fd:
            all_modified = yaml.safe_load(expected_modified_fd)
        try:
            expected_modified = all_modified['%s_%s' % (prod, ver)]
        except KeyError:
            try:
                expected_modified = all_modified['%s_%s' % (plat, ver)]
            except KeyError:
                self.log.append({
                    'result': 'skip',
                    'comment': 'unsupported region and/or platform-version combination'})
                return self.log

        ret = {
            'expected modified files': expected_modified,
            'actual modified files': modified_files
        }

        ret['result'] = expected_modified == modified_files and 'passed' or 'failed'
        self.log.append(ret)

        return self.log



""" This module contains testcase_10_networking test """
from testcase import Testcase


class testcase_10_networking(Testcase):
    """
    Check for networking setup
    """

    stages = ['stage1']
    tags = ['default']

    # pylint: disable=W0613
    def test(self, connection, params):
        """ Perform test """

        self.get_return_value(connection, 'grep "^NETWORKING=yes" /etc/sysconfig/network')
        self.get_return_value(connection, 'egrep "^DEVICE=(|\\\")eth0(|\\\")" /etc/sysconfig/network-scripts/ifcfg-eth0')
        return self.log



""" This module contains testcase_39_root_is_locked test """
from testcase import Testcase


class testcase_39_root_is_locked(Testcase):
    """
    Root account should be locked
    """
    tags = ['default']
    stages = ['stage1']

    # pylint: disable=W0613
    def test(self, connection, params):
        """ Perform test """

        self.get_return_value(connection, 'passwd -S root | grep -q LK')
        return self.log



""" This module contains testcase_14_host_details test """
from testcase import Testcase
from dva.work.common import RESULT_FAILED
import json
import re


class testcase_14_host_details(Testcase):
    """
    Try to fetch host details from EC2 and compare with expectation
    """
    stages = ['stage1']
    tags = ['default']
    applicable = {'cloud': 'ec2'}

    def test(self, connection, params):
        """ Perform test """

        prod = params['platform'].upper()
        self.get_return_value(connection, '[ ! -z \'`curl http://169.254.169.254/latest/dynamic/instance-identity/signature`\' ]')
        json_str = self.match(connection, 'curl http://169.254.169.254/latest/dynamic/instance-identity/document', re.compile('.*({.*}).*', re.DOTALL))
        if json_str:
            try:
                jstruct = json.loads(json_str[0])
                if 'billingProducts' in jstruct.keys() and not jstruct['billingProducts'] is None:
                    billing_platform = jstruct['billingProducts'][0]
                else:
                    billing_platform = ''
                self.get_return_value(connection, '[ \'%s\' = \'%s\' ]' % (jstruct['imageId'], params['ami']))
                self.get_return_value(connection, '[ \'%s\' = \'%s\' ]' % (jstruct['architecture'], params['arch']))
                self.get_return_value(connection, '[ \'%s\' = \'%s\' ]' % (jstruct['region'], params['region']))
                if prod in ['RHEL', 'BETA']:
                    if params['itype'] == 'hourly':
                        self.get_return_value(connection, '[ \'%s\' = \'%s\' ]' % (billing_platform, 'bp-6fa54006'))
                    elif params['itype'] == 'access':
                        self.get_return_value(connection, '[ \'%s\' = \'%s\' ]' % (billing_platform, 'bp-63a5400a'))
            except KeyError as exc:
                self.log.append({'result': RESULT_FAILED, 'comment': 'failed to check instance details, ' + exc.message})
        else:
            self.log.append({'result': RESULT_FAILED, 'comment': 'failed to get instance details'})
        return self.log



""" This module contains testcase_05_grub test """

from testcase import Testcase


class testcase_05_grub(Testcase):
    """
    Check grub config:
    - /boot/grub/menu.lst exists
    - /boot/grub/menu.lst is symlink for /boot/grub/grub.conf
    - hard drive is not (hd0,0) for paravirtual
    """
    stages = ['stage1']
    applicable = {'platform': '(?i)RHEL|BETA', 'version': 'OS (>=5.5, <7.0)'}
    tags = ['default']

    def test(self, connection, params):
        """ Perform test """

        self.get_return_value(connection, 'test -h /boot/grub/menu.lst')
        self.ping_pong(connection, 'readlink -e /boot/grub/menu.lst', '/boot/grub/grub.conf')
        if params['virtualization'] != 'hvm':
            self.get_return_value(connection, 'grep \'(hd0,0)\' /boot/grub/grub.conf', expected_status=1)
        return self.log



""" This module contains testcase_30_rhn_certificates test """
from testcase import Testcase
from datetime import datetime
from distutils.version import LooseVersion as Version
from dva.work.common import RESULT_FAILED
import re


def _expiration_date(params):
    """ Get expiration delta in years """
    seven_year_releases = lambda ver: ver >= Version('5.5') and ver <= Version('5.8') or\
                                ver >= Version('6.0') and ver <= Version('6.5')


    if seven_year_releases(params['version']):
        expiration = 7
    else:
        expiration = 10
    version = Version(params['version'])
    if version < '6.0':
        return datetime(2007 + expiration, 3, 14)
    if version < '7.0':
        return datetime(2010 + expiration, 11, 10)
    if version < '8.0':
        return datetime(2014 + expiration, 6, 10)
    raise ValueError('release %s unsupported' % params['version'])


class testcase_30_rhn_certificates(Testcase):
    """
    Check for rhn certificates lifetime
    """
    stages = ['stage1']
    applicable = {'platform': '(?i)RHEL|BETA', 'version': 'OS (>= 5.5)'}
    tags = ['default']

    def test(self, connection, params):
        """ Perform test """

        if params['platform'].upper() == 'BETA':
            config_rpms = 'rh-amazon-rhui-client rh-amazon-rhui-client-beta'
        else:
            config_rpms = 'rh-amazon-rhui-client'
        cert_files = [cert_file for cert_file in self.get_result(connection, "rpm -ql " + config_rpms).split() \
                        if re.match(r'.*\.(crt|pem)$', cert_file)]
        # for each cert file, the notAfter field is examined
        # against the expiration_date and the result is stored in self.log
        try:
            expiration_date = _expiration_date(params)
        except ValueError as err:
            # just log and return in case expiration can't be determined
            self.log.append({'result': RESULT_FAILED, 'comment': str(err)})
            return self.log
        for cert in cert_files:
            # the -dates format is:
            # notBefore=Jan 17 10:29:59 2013 GMT
            # notAfter=Jan 17 10:29:59 2014 GMT
            # just the notAfter line is required
            date_string = re.split(
                r'\r?\n',
                self.get_result(connection, 'openssl x509 -in %s -noout -dates' % cert).strip()
            )[-1]
            if date_string and date_string.find('=') != -1:
                cert_expiration = datetime.strptime(
                    # the date_string has the form:
                    #   notAfter=Nov 11 00:00:00 2020 GMT
                    date_string.split('=')[1],
                    '%b %d %H:%M:%S %Y %Z'
                )
                self.log.append(
                    {
                        'result': expiration_date <= cert_expiration
                        and 'passed' or 'failed',
                        'comment': '(%s).notAfter=%s; expecting: %s' %
                        (cert, cert_expiration, expiration_date)
                    }
                )
            else:
                self.log.append({'result': 'failed', 'comment': 'failed to check expiration date for  %s' % cert})

        return self.log
